@@ -35,8 +35,8 @@ Available unstable (nightly-only) flags:
     -Z unstable-options -- Allow the usage of unstable options such as --registry
     -Z config-profile   -- Read profiles from .cargo/config files
     -Z install-upgrade  -- `cargo install` will upgrade instead of failing
-    -Z cache-messages   -- Cache compiler messages
     -Z timings          -- Display concurrency information
+    -Z doctest-xcompile -- Compile and run doctests for non-host target using runner config
 
 Run with 'cargo -Z [FLAG] [SUBCOMMAND]'"
         );
