@@ -1,14 +1,11 @@
 use std::fs::File;
 
-use git2;
-
-use crate::support::git;
-use crate::support::is_nightly;
-use crate::support::{basic_manifest, project};
+use cargo_test_support::git;
+use cargo_test_support::{basic_manifest, clippy_is_available, project};
 
 use std::io::Write;
 
-#[test]
+#[cargo_test]
 fn do_not_fix_broken_builds() {
     let p = project()
         .file(
@@ -29,12 +26,12 @@ fn do_not_fix_broken_builds() {
     p.cargo("fix --allow-no-vcs")
         .env("__CARGO_FIX_YOLO", "1")
         .with_status(101)
-        .with_stderr_contains("[ERROR] Could not compile `foo`.")
+        .with_stderr_contains("[ERROR] could not compile `foo`.")
         .run();
     assert!(p.read_file("src/lib.rs").contains("let mut x = 3;"));
 }
 
-#[test]
+#[cargo_test]
 fn fix_broken_if_requested() {
     let p = project()
         .file(
@@ -53,7 +50,7 @@ fn fix_broken_if_requested() {
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn broken_fixes_backed_out() {
     // This works as follows:
     // - Create a `rustc` shim (the "foo" project) which will pretend that the
@@ -134,16 +131,15 @@ fn broken_fixes_backed_out() {
         .build();
 
     // Build our rustc shim
-    p.cargo("build").cwd(p.root().join("foo")).run();
+    p.cargo("build").cwd("foo").run();
 
     // Attempt to fix code, but our shim will always fail the second compile
     p.cargo("fix --allow-no-vcs --lib")
-        .cwd(p.root().join("bar"))
+        .cwd("bar")
         .env("__CARGO_FIX_YOLO", "1")
         .env("RUSTC", p.root().join("foo/target/debug/foo"))
         .with_stderr_contains(
-            "\
-             warning: failed to automatically apply fixes suggested by rustc \
+            "warning: failed to automatically apply fixes suggested by rustc \
              to crate `bar`\n\
              \n\
              after fixes were automatically applied the compiler reported \
@@ -173,7 +169,7 @@ fn broken_fixes_backed_out() {
     assert!(p.read_file("bar/src/lib.rs").contains("let mut x = 3;"));
 }
 
-#[test]
+#[cargo_test]
 fn fix_path_deps() {
     let p = project()
         .file(
@@ -227,7 +223,7 @@ fn fix_path_deps() {
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn do_not_fix_non_relevant_deps() {
     let p = project()
         .no_manifest()
@@ -259,23 +255,19 @@ fn do_not_fix_non_relevant_deps() {
 
     p.cargo("fix --allow-no-vcs")
         .env("__CARGO_FIX_YOLO", "1")
-        .cwd(p.root().join("foo"))
+        .cwd("foo")
         .run();
 
     assert!(p.read_file("bar/src/lib.rs").contains("mut"));
 }
 
-#[test]
+#[cargo_test]
 fn prepare_for_2018() {
-    if !is_nightly() {
-        return;
-    }
     let p = project()
         .file(
             "src/lib.rs",
             r#"
                 #![allow(unused)]
-                #![feature(rust_2018_preview)]
 
                 mod foo {
                     pub const FOO: &str = "fooo";
@@ -309,17 +301,12 @@ fn prepare_for_2018() {
         .contains("let x = crate::foo::FOO;"));
 }
 
-#[test]
+#[cargo_test]
 fn local_paths() {
-    if !is_nightly() {
-        return;
-    }
     let p = project()
         .file(
             "src/lib.rs",
             r#"
-                #![feature(rust_2018_preview)]
-
                 use test::foo;
 
                 mod test {
@@ -348,11 +335,8 @@ fn local_paths() {
     assert!(p.read_file("src/lib.rs").contains("use crate::test::foo;"));
 }
 
-#[test]
+#[cargo_test]
 fn upgrade_extern_crate() {
-    if !is_nightly() {
-        return;
-    }
     let p = project()
         .file(
             "Cargo.toml",
@@ -401,17 +385,13 @@ fn upgrade_extern_crate() {
     assert!(!p.read_file("src/lib.rs").contains("extern crate"));
 }
 
-#[test]
+#[cargo_test]
 fn specify_rustflags() {
-    if !is_nightly() {
-        return;
-    }
     let p = project()
         .file(
             "src/lib.rs",
             r#"
                 #![allow(unused)]
-                #![feature(rust_2018_preview)]
 
                 mod foo {
                     pub const FOO: &str = "fooo";
@@ -430,13 +410,13 @@ fn specify_rustflags() {
 [FINISHED] [..]
 ";
     p.cargo("fix --edition --allow-no-vcs")
-        .env("RUSTFLAGS", "-C target-cpu=native")
+        .env("RUSTFLAGS", "-C linker=cc")
         .with_stderr(stderr)
         .with_stdout("")
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn no_changes_necessary() {
     let p = project().file("src/lib.rs", "").build();
 
@@ -450,7 +430,7 @@ fn no_changes_necessary() {
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn fixes_extra_mut() {
     let p = project()
         .file(
@@ -476,7 +456,7 @@ fn fixes_extra_mut() {
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn fixes_two_missing_ampersands() {
     let p = project()
         .file(
@@ -503,7 +483,7 @@ fn fixes_two_missing_ampersands() {
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn tricky() {
     let p = project()
         .file(
@@ -529,13 +509,12 @@ fn tricky() {
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn preserve_line_endings() {
     let p = project()
         .file(
             "src/lib.rs",
-            "\
-             fn add(a: &u32) -> u32 { a + 1 }\r\n\
+            "fn add(a: &u32) -> u32 { a + 1 }\r\n\
              pub fn foo() -> u32 { let mut x = 3; add(&x) }\r\n\
              ",
         )
@@ -547,14 +526,13 @@ fn preserve_line_endings() {
     assert!(p.read_file("src/lib.rs").contains("\r\n"));
 }
 
-#[test]
+#[cargo_test]
 fn fix_deny_warnings() {
     let p = project()
         .file(
             "src/lib.rs",
-            "\
-                #![deny(warnings)]
-                pub fn foo() { let mut x = 3; drop(x); }
+            "#![deny(warnings)]
+             pub fn foo() { let mut x = 3; drop(x); }
             ",
         )
         .build();
@@ -564,7 +542,7 @@ fn fix_deny_warnings() {
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn fix_deny_warnings_but_not_others() {
     let p = project()
         .file(
@@ -589,7 +567,7 @@ fn fix_deny_warnings_but_not_others() {
     assert!(p.read_file("src/lib.rs").contains("fn bar() {}"));
 }
 
-#[test]
+#[cargo_test]
 fn fix_two_files() {
     let p = project()
         .file(
@@ -624,7 +602,7 @@ fn fix_two_files() {
     assert!(!p.read_file("src/bar.rs").contains("let mut x = 3;"));
 }
 
-#[test]
+#[cargo_test]
 fn fixes_missing_ampersand() {
     let p = project()
         .file("src/main.rs", "fn main() { let mut x = 3; drop(x); }")
@@ -668,7 +646,7 @@ fn fixes_missing_ampersand() {
     p.cargo("test").run();
 }
 
-#[test]
+#[cargo_test]
 fn fix_features() {
     let p = project()
         .file(
@@ -699,10 +677,13 @@ fn fix_features() {
     p.cargo("build --features bar").run();
 }
 
-#[test]
+#[cargo_test]
 fn shows_warnings() {
     let p = project()
-        .file("src/lib.rs", "#[deprecated] fn bar() {} pub fn foo() { let _ = bar(); }")
+        .file(
+            "src/lib.rs",
+            "#[deprecated] fn bar() {} pub fn foo() { let _ = bar(); }",
+        )
         .build();
 
     p.cargo("fix --allow-no-vcs")
@@ -710,15 +691,14 @@ fn shows_warnings() {
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn warns_if_no_vcs_detected() {
     let p = project().file("src/lib.rs", "pub fn foo() {}").build();
 
     p.cargo("fix")
         .with_status(101)
         .with_stderr(
-            "\
-             error: no VCS found for this package and `cargo fix` can potentially perform \
+            "error: no VCS found for this package and `cargo fix` can potentially perform \
              destructive changes; if you'd like to suppress this error pass `--allow-no-vcs`\
              ",
         )
@@ -726,17 +706,10 @@ fn warns_if_no_vcs_detected() {
     p.cargo("fix --allow-no-vcs").run();
 }
 
-#[test]
+#[cargo_test]
 fn warns_about_dirty_working_directory() {
-    let p = project().file("src/lib.rs", "pub fn foo() {}").build();
+    let p = git::new("foo", |p| p.file("src/lib.rs", "pub fn foo() {}"));
 
-    let repo = git2::Repository::init(&p.root()).unwrap();
-    let mut cfg = t!(repo.config());
-    t!(cfg.set_str("user.email", "foo@bar.com"));
-    t!(cfg.set_str("user.name", "Foo Bar"));
-    drop(cfg);
-    git::add(&repo);
-    git::commit(&repo);
     File::create(p.root().join("src/lib.rs")).unwrap();
 
     p.cargo("fix")
@@ -757,17 +730,10 @@ commit the changes to these files:
     p.cargo("fix --allow-dirty").run();
 }
 
-#[test]
+#[cargo_test]
 fn warns_about_staged_working_directory() {
-    let p = project().file("src/lib.rs", "pub fn foo() {}").build();
+    let (p, repo) = git::new_repo("foo", |p| p.file("src/lib.rs", "pub fn foo() {}"));
 
-    let repo = git2::Repository::init(&p.root()).unwrap();
-    let mut cfg = t!(repo.config());
-    t!(cfg.set_str("user.email", "foo@bar.com"));
-    t!(cfg.set_str("user.name", "Foo Bar"));
-    drop(cfg);
-    git::add(&repo);
-    git::commit(&repo);
     File::create(&p.root().join("src/lib.rs"))
         .unwrap()
         .write_all("pub fn bar() {}".to_string().as_bytes())
@@ -792,41 +758,25 @@ commit the changes to these files:
     p.cargo("fix --allow-staged").run();
 }
 
-#[test]
+#[cargo_test]
 fn does_not_warn_about_clean_working_directory() {
-    let p = project().file("src/lib.rs", "pub fn foo() {}").build();
-
-    let repo = git2::Repository::init(&p.root()).unwrap();
-    let mut cfg = t!(repo.config());
-    t!(cfg.set_str("user.email", "foo@bar.com"));
-    t!(cfg.set_str("user.name", "Foo Bar"));
-    drop(cfg);
-    git::add(&repo);
-    git::commit(&repo);
-
+    let p = git::new("foo", |p| p.file("src/lib.rs", "pub fn foo() {}"));
     p.cargo("fix").run();
 }
 
-#[test]
+#[cargo_test]
 fn does_not_warn_about_dirty_ignored_files() {
-    let p = project()
-        .file("src/lib.rs", "pub fn foo() {}")
-        .file(".gitignore", "bar\n")
-        .build();
+    let p = git::new("foo", |p| {
+        p.file("src/lib.rs", "pub fn foo() {}")
+            .file(".gitignore", "bar\n")
+    });
 
-    let repo = git2::Repository::init(&p.root()).unwrap();
-    let mut cfg = t!(repo.config());
-    t!(cfg.set_str("user.email", "foo@bar.com"));
-    t!(cfg.set_str("user.name", "Foo Bar"));
-    drop(cfg);
-    git::add(&repo);
-    git::commit(&repo);
     File::create(p.root().join("bar")).unwrap();
 
     p.cargo("fix").run();
 }
 
-#[test]
+#[cargo_test]
 fn fix_all_targets_by_default() {
     let p = project()
         .file("src/lib.rs", "pub fn foo() { let mut x = 3; drop(x); }")
@@ -839,7 +789,7 @@ fn fix_all_targets_by_default() {
     assert!(!p.read_file("tests/foo.rs").contains("let mut x"));
 }
 
-#[test]
+#[cargo_test]
 fn prepare_for_and_enable() {
     let p = project()
         .file(
@@ -872,17 +822,12 @@ information about transitioning to the 2018 edition see:
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn fix_overlapping() {
-    if !is_nightly() {
-        return;
-    }
     let p = project()
         .file(
             "src/lib.rs",
             r#"
-                #![feature(rust_2018_preview)]
-
                 pub fn foo<T>() {}
                 pub struct A;
 
@@ -910,11 +855,8 @@ fn fix_overlapping() {
     assert!(contents.contains("crate::foo::<crate::A>()"));
 }
 
-#[test]
+#[cargo_test]
 fn fix_idioms() {
-    if !is_nightly() {
-        return;
-    }
     let p = project()
         .file(
             "Cargo.toml",
@@ -948,16 +890,14 @@ fn fix_idioms() {
     assert!(p.read_file("src/lib.rs").contains("Box<dyn Any>"));
 }
 
-#[test]
+#[cargo_test]
 fn idioms_2015_ok() {
     let p = project().file("src/lib.rs", "").build();
 
-    p.cargo("fix --edition-idioms --allow-no-vcs")
-        .masquerade_as_nightly_cargo()
-        .run();
+    p.cargo("fix --edition-idioms --allow-no-vcs").run();
 }
 
-#[test]
+#[cargo_test]
 fn both_edition_migrate_flags() {
     let p = project().file("src/lib.rs", "").build();
 
@@ -965,7 +905,7 @@ fn both_edition_migrate_flags() {
 error: The argument '--edition' cannot be used with '--prepare-for <prepare-for>'
 
 USAGE:
-    cargo[..] fix --edition --message-format <FMT>
+    cargo[..] fix --edition
 
 For more information try --help
 ";
@@ -976,7 +916,7 @@ For more information try --help
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn shows_warnings_on_second_run_without_changes() {
     let p = project()
         .file(
@@ -1001,7 +941,7 @@ fn shows_warnings_on_second_run_without_changes() {
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn shows_warnings_on_second_run_without_changes_on_multiple_targets() {
     let p = project()
         .file(
@@ -1080,7 +1020,7 @@ fn shows_warnings_on_second_run_without_changes_on_multiple_targets() {
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn doesnt_rebuild_dependencies() {
     let p = project()
         .file(
@@ -1125,7 +1065,7 @@ fn doesnt_rebuild_dependencies() {
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn does_not_crash_with_rustc_wrapper() {
     // We don't have /usr/bin/env on Windows.
     if cfg!(windows) {
@@ -1148,7 +1088,7 @@ fn does_not_crash_with_rustc_wrapper() {
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn only_warn_for_relevant_crates() {
     let p = project()
         .file(
@@ -1194,11 +1134,8 @@ fn only_warn_for_relevant_crates() {
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn fix_to_broken_code() {
-    if !is_nightly() {
-        return;
-    }
     let p = project()
         .file(
             "foo/Cargo.toml",
@@ -1254,11 +1191,11 @@ fn fix_to_broken_code() {
         .build();
 
     // Build our rustc shim
-    p.cargo("build").cwd(p.root().join("foo")).run();
+    p.cargo("build").cwd("foo").run();
 
     // Attempt to fix code, but our shim will always fail the second compile
     p.cargo("fix --allow-no-vcs --broken-code")
-        .cwd(p.root().join("bar"))
+        .cwd("bar")
         .env("RUSTC", p.root().join("foo/target/debug/foo"))
         .with_status(101)
         .with_stderr_contains("[WARNING] failed to automatically apply fixes [..]")
@@ -1270,7 +1207,7 @@ fn fix_to_broken_code() {
     );
 }
 
-#[test]
+#[cargo_test]
 fn fix_with_common() {
     let p = project()
         .file("src/lib.rs", "")
@@ -1289,3 +1226,85 @@ fn fix_with_common() {
 
     assert_eq!(p.read_file("tests/common/mod.rs"), "pub fn r#try() {}");
 }
+
+#[cargo_test]
+fn fix_in_existing_repo_weird_ignore() {
+    // Check that ignore doesn't ignore the repo itself.
+    let p = git::new("foo", |project| {
+        project
+            .file("src/lib.rs", "")
+            .file(".gitignore", "foo\ninner\n")
+            .file("inner/file", "")
+    });
+
+    p.cargo("fix").run();
+    // This is questionable about whether it is the right behavior. It should
+    // probably be checking if any source file for the current project is
+    // ignored.
+    p.cargo("fix")
+        .cwd("inner")
+        .with_stderr_contains("[ERROR] no VCS found[..]")
+        .with_status(101)
+        .run();
+    p.cargo("fix").cwd("src").run();
+}
+
+#[cargo_test]
+fn fix_with_clippy() {
+    if !clippy_is_available() {
+        return;
+    }
+
+    let p = project()
+        .file(
+            "src/lib.rs",
+            "
+                pub fn foo() {
+                    let mut v = Vec::<String>::new();
+                    let _ = v.iter_mut().filter(|&ref a| a.is_empty());
+                }
+    ",
+        )
+        .build();
+
+    let stderr = "\
+[CHECKING] foo v0.0.1 ([..])
+[FIXING] src/lib.rs (1 fix)
+[FINISHED] [..]
+";
+
+    p.cargo("fix -Zunstable-options --clippy --allow-no-vcs")
+        .masquerade_as_nightly_cargo()
+        .with_stderr(stderr)
+        .with_stdout("")
+        .run();
+
+    assert_eq!(
+        p.read_file("src/lib.rs"),
+        "
+                pub fn foo() {
+                    let mut v = Vec::<String>::new();
+                    let _ = v.iter_mut().filter(|a| a.is_empty());
+                }
+    "
+    );
+}
+
+#[cargo_test]
+fn fix_color_message() {
+    // Check that color appears in diagnostics.
+    let p = project()
+        .file("src/lib.rs", "std::compile_error!{\"color test\"}")
+        .build();
+
+    p.cargo("fix --allow-no-vcs --color=always")
+        .with_stderr_contains("[..]\x1b[[..]")
+        .with_status(101)
+        .run();
+
+    p.cargo("fix --allow-no-vcs --color=never")
+        .with_stderr_contains("error: color test")
+        .with_stderr_does_not_contain("[..]\x1b[[..]")
+        .with_status(101)
+        .run();
+}
