@@ -1,5 +1,5 @@
-use crate::support::{basic_bin_manifest, basic_lib_manifest, project, Project};
 use cargo::util::paths::dylib_path_envvar;
+use cargo_test_support::{basic_bin_manifest, basic_lib_manifest, project, Project};
 
 #[cargo_test]
 fn simple() {
@@ -628,14 +628,14 @@ fn example_with_release_flag() {
         .with_stderr(
             "\
 [COMPILING] bar v0.5.0 ([CWD]/bar)
-[RUNNING] `rustc --crate-name bar bar/src/bar.rs --color never --crate-type lib \
+[RUNNING] `rustc --crate-name bar bar/src/bar.rs [..]--crate-type lib \
         --emit=[..]link \
         -C opt-level=3 \
         -C metadata=[..] \
         --out-dir [CWD]/target/release/deps \
         -L dependency=[CWD]/target/release/deps`
 [COMPILING] foo v0.0.1 ([CWD])
-[RUNNING] `rustc --crate-name a examples/a.rs --color never --crate-type bin \
+[RUNNING] `rustc --crate-name a examples/a.rs [..]--crate-type bin \
         --emit=[..]link \
         -C opt-level=3 \
         -C metadata=[..] \
@@ -657,14 +657,14 @@ fast2",
         .with_stderr(
             "\
 [COMPILING] bar v0.5.0 ([CWD]/bar)
-[RUNNING] `rustc --crate-name bar bar/src/bar.rs --color never --crate-type lib \
+[RUNNING] `rustc --crate-name bar bar/src/bar.rs [..]--crate-type lib \
         --emit=[..]link \
         -C debuginfo=2 \
         -C metadata=[..] \
         --out-dir [CWD]/target/debug/deps \
         -L dependency=[CWD]/target/debug/deps`
 [COMPILING] foo v0.0.1 ([CWD])
-[RUNNING] `rustc --crate-name a examples/a.rs --color never --crate-type bin \
+[RUNNING] `rustc --crate-name a examples/a.rs [..]--crate-type bin \
         --emit=[..]link \
         -C debuginfo=2 \
         -C metadata=[..] \
@@ -797,8 +797,7 @@ fn run_from_executable_folder() {
     p.cargo("run")
         .cwd(cwd)
         .with_stderr(
-            "\
-             [FINISHED] dev [unoptimized + debuginfo] target(s) in [..]\n\
+            "[FINISHED] dev [unoptimized + debuginfo] target(s) in [..]\n\
              [RUNNING] `./foo[EXE]`",
         )
         .with_stdout("hello")
@@ -1072,7 +1071,7 @@ fn default_run_workspace() {
 #[cargo_test]
 #[cfg(target_os = "macos")]
 fn run_link_system_path_macos() {
-    use crate::support::paths::{self, CargoPathExt};
+    use cargo_test_support::paths::{self, CargoPathExt};
     use std::fs;
     // Check that the default system library path is honored.
     // First, build a shared library that will be accessed from
